package remote

import (
	"context"
	"errors"
	"strings"
	"testing"

	"io/ioutil"
	"net/http"
)

type TestRoundTripper struct {
	StatusCode int
	Body       string
}

func (t *TestRoundTripper) RoundTrip(r *http.Request) (*http.Response, error) {
	if r.Header.Get("Fastly-Key") == "" {
		return nil, errors.New("Fastly-Key header is required")
	}

	return &http.Response{
		StatusCode: t.StatusCode,
		Body:       ioutil.NopCloser(strings.NewReader(t.Body)),
	}, nil
}

func TestLatestVersion(t *testing.T) {
	c := NewFastlyClient(&http.Client{
		Transport: &TestRoundTripper{
			StatusCode: 200,
			Body: `
{
	"created_at": "2020-04-09T18:14:30.000Z",
	"updated_at": "2020-04-09T18:15:30.000Z",
	"deleted_at": null,
	"active": true,
	"comment": "",
	"deployed": true,
	"locked": false,
	"number": 1,
	"staging": false,
	"testing": false,
	"service_id": "SU1Z0isxPaozGVKXdv0eY"
}`,
		},
	}, "dummy", "dummy")

	version, err := c.LatestVersion(context.Background())
	if err != nil {
		t.Errorf("Unexpected error: %s", err)
		t.FailNow()
	}
	if version != 1 {
		t.Errorf("Version assertion error, expects=1 but got=%d", version)
		t.FailNow()
	}
}

func TestListDictionaries(t *testing.T) {
	c := NewFastlyClient(&http.Client{
		Transport: &TestRoundTripper{
			StatusCode: 200,
			Body: `
[
  {
	"created_at": "2020-04-29T22:16:23.000Z",
	"deleted_at": null,
	"id": "3vjTN8v1O7nOAY7aNDGOL",
	"name": "my_dictionary",
	"service_id": "SU1Z0isxPaozGVKXdv0eY",
	"updated_at": "2020-04-29T22:16:23.000Z",
	"version": 1,
	"write_only": false
  }
]`,
		},
	}, "dummy", "dummy")

	dicts, err := c.ListEdgeDictionaries(context.Background(), 10)
	if err != nil {
		t.Errorf("Unexpected error: %s", err)
		t.FailNow()
	}
	if len(dicts) != 1 {
		t.Errorf("dictionaries should have 1 but got %d", len(dicts))
		t.FailNow()
	}
	d := dicts[0]
	if d.Id != "3vjTN8v1O7nOAY7aNDGOL" {
		t.Errorf("dict id assertion error, expects=3vjTN8v1O7nOAY7aNDGOL but got=%s", d.Id)
		t.FailNow()
	}
	if d.Name != "my_dictionary" {
		t.Errorf("dict name assertion error, expects=my_dictionary but got=%s", d.Name)
		t.FailNow()
	}
}

func TestListDictionaryItems(t *testing.T) {
	c := NewFastlyClient(&http.Client{
		Transport: &TestRoundTripper{
			StatusCode: 200,
			Body: `
[
  {
	"dictionary_id": "3vjTN8v1O7nOAY7aNDGOL",
	"service_id": "SU1Z0isxPaozGVKXdv0eY",
	"item_key": "some_key",
	"item_value": "some_value",
	"created_at": "2020-04-21T18:14:32.000Z",
	"deleted_at": null,
	"updated_at": "2020-04-21T18:14:32.000Z"
  }
]`,
		},
	}, "dummy", "dummy")

	items, err := c.ListEdgeDictionaryItems(context.Background(), "3vjTN8v1O7nOAY7aNDGOL")
	if err != nil {
		t.Errorf("Unexpected error: %s", err)
		t.FailNow()
	}
	if len(items) != 1 {
		t.Errorf("dictionaries should have 1 items but got %d", len(items))
		t.FailNow()
	}
	i := items[0]
	if i.Key != "some_key" {
		t.Errorf("item key assertion error, expects=some_key but got=%s", i.Key)
		t.FailNow()
	}
	if i.Value != "some_value" {
		t.Errorf("item value assertion error, expects=some_value but got=%s", i.Value)
		t.FailNow()
	}
}

func TestListAccessControlLists(t *testing.T) {
	c := NewFastlyClient(&http.Client{
		Transport: &TestRoundTripper{
			StatusCode: 200,
			Body: `
[
  {
    "name": "blocked_ips",
    "service_id": "0yGwmmav8rcXRC7yRwzPNQ",
    "id": "1GJz4D4wxiP8DeVCVdYDfo",
    "deleted_at": null,
    "created_at": "2021-11-25T23:40:04Z",
    "updated_at": "2021-11-25T23:40:04Z",
    "version": "10"
  }
]]`,
		},
	}, "dummy", "dummy")

	acls, err := c.ListAccessControlLists(context.Background(), 10)
	if err != nil {
		t.Errorf("Unexpected error: %s", err)
		t.FailNow()
	}
	if len(acls) != 1 {
		t.Errorf("dictionaries should have 1 but got %d", len(acls))
		t.FailNow()
	}
	a := acls[0]
	if a.Id != "1GJz4D4wxiP8DeVCVdYDfo" {
		t.Errorf("acl id assertion error, expects=1GJz4D4wxiP8DeVCVdYDfo but got=%s", a.Id)
		t.FailNow()
	}
	if a.Name != "blocked_ips" {
		t.Errorf("acl name assertion error, expects=blocked_ips but got=%s", a.Name)
		t.FailNow()
	}
}

func TestListAccessControlEntiries(t *testing.T) {
	c := NewFastlyClient(&http.Client{
		Transport: &TestRoundTripper{
			StatusCode: 200,
			Body: `
[
  {
    "updated_at": "2021-11-25T23:40:33Z",
    "ip": "10.0.0.0",
    "negated": "0",
    "acl_id": "1GJz4D4wxiP8DeVCVdYDfo",
    "id": "4BCmH8eb7p9absKKcaADIp",
    "subnet": 32,
    "service_id": "0yGwmmav8rcXRC7yRwzPNQ",
    "comment": "example",
    "created_at": "2021-11-25T23:40:33Z"
  }
]`,
		},
	}, "dummy", "dummy")

	items, err := c.ListAccessControlEntries(context.Background(), "1GJz4D4wxiP8DeVCVdYDfo")
	if err != nil {
		t.Errorf("Unexpected error: %s", err)
		t.FailNow()
	}
	if len(items) != 1 {
		t.Errorf("entries should have 1 items but got %d", len(items))
		t.FailNow()
	}
	i := items[0]
	if i.Ip != "10.0.0.0" {
		t.Errorf("ip assertion error, expects=10.0.0.0 but got=%s", i.Ip)
		t.FailNow()
	}
	if i.Negated != "0" {
		t.Errorf("negated field assertion error, expects=0 but got=%s", i.Negated)
		t.FailNow()
	}
	if *i.Subnet != 32 {
		t.Errorf("subnet field assertion error, expects=32 but got=%d", *i.Subnet)
		t.FailNow()
	}
}
